﻿local JDI = JDI_2D07EA85_B774_43bf_873C_67DFA1FFDCA1;

local MainFrame = JDI.GUI.MainFrame;
MainFrame:DoInit();

MainFrame:RegisterEvent("PLAYER_LOGIN");
MainFrame:RegisterEvent("UNIT_INVENTORY_CHANGED");

local GetItemInfo, GetContainerNumSlots, GetContainerItemLink, GetInventoryItemLink, UnitIsEnemy, UnitInVehicle =
      GetItemInfo, GetContainerNumSlots, GetContainerItemLink, GetInventoryItemLink, UnitIsEnemy, UnitInVehicle;

function JDI:PLAYER_LOGIN()
    if (not JoustDoItDB) then
        JoustDoItDB = {};
    end
    
    JDI.CharName = (UnitName("player")) .. " - " .. (GetRealmName());
    if (JoustDoItDB[JDI.CharName]) then
        JDI.Vars = JoustDoItDB[JDI.CharName];
    else
        JoustDoItDB[JDI.CharName] = {};
        JDI.Vars = JoustDoItDB[JDI.CharName];
    end
    
    -- Defaults
    if (JDI.Vars.Enable == nil) then
        JDI.Vars.Enable = true;
    end
    if (JDI.Vars.Combat == nil) then
        JDI.Vars.Combat = false;
    end
    if (JDI.Vars.Dismount == nil) then
        JDI.Vars.Dismount = true;
    end

--  Removed in favor of the new fail-safe which better fits 3.3
    JDI.Vars.Target = nil;
--[[if (JDI.Vars.Target == nil) then
       JDI.Vars.Target = true;
    end]]
    
    if (JDI.Vars.Enable) then
        MainFrame:RegisterEvent("UI_ERROR_MESSAGE");
        MainFrame:RegisterEvent("UNIT_EXITED_VEHICLE");
        if (JDI.Vars.Combat) then
            MainFrame:RegisterEvent("PLAYER_REGEN_DISABLED");
        end
    end
    
    JDI:UNIT_INVENTORY_CHANGED();
end

function JDI:UNIT_INVENTORY_CHANGED()
    local link = GetInventoryItemLink("player", JDI.MAINHANDSLOT);
    if (link) then
        if (not JDI:IsLance(link)) then
            JDI.Vars.MainHand = link;
            JDI.Vars.OffHand = GetInventoryItemLink("player", JDI.OFFHANDSLOT);
        end
    end
end

function JDI:UI_ERROR_MESSAGE(msg)
    if (msg == SPELL_FAILED_CUSTOM_ERROR_60) then
        if (InCombatLockdown()) then
            -- It doesn't make sense to queue Lance swapping, by the time the player leaves combat they won't need it
            -- Tell the player to swap manually, automatic swapping back to weapons will still work.
            JDI:Warn(JDI.Locale.InCombatLance, "Sound\\Spells\\PVPFlagTaken.wav");
        else
            JDI:SwapToLance(); -- Don't queue, we need it NAO!!!
        end
    end
end

function JDI:UNIT_EXITED_VEHICLE(unit)
    local link = GetInventoryItemLink("player", JDI.MAINHANDSLOT);
    if ((link and JDI:IsLance(link)) and (unit == "player")) then
        JDI:QueueSwap("SwapToWeapons");
    end
end

function JDI:PLAYER_REGEN_DISABLED()
    local link = GetInventoryItemLink("player", JDI.MAINHANDSLOT);
    if (link and JDI:IsLance(link) and not UnitInVehicle("player")) then
        JDI:SwapToWeapons(); -- Don't queue, we need it NAO!!!
    end
end

--[[ x87bliss
    The Queue functions were originally implemented as a table with time codes by Dridzt
    However, since only the latest 'queued' function is required, I changed this to a single variable.
    This variable will hold whatever the latest call to QueueSwap was.

    Currently the only function queuable is SwapToWeapons. SwapQueue could be changed to a boolean variable
    if no other function needs to be queued - wait til next version to make sure no bugs.
  ]]
local SwapQueue;

function JDI:PLAYER_REGEN_ENABLED()
    if (SwapQueue) then
        JDI[SwapQueue](self);
    end
    SwapQueue = nil; -- Clear any previously queued function
    MainFrame:UnregisterEvent("PLAYER_REGEN_ENABLED");
end

function JDI:QueueSwap(func)
    if (InCombatLockdown()) then
        SwapQueue = func;
        if (not MainFrame:IsEventRegistered("PLAYER_REGEN_ENABLED")) then
            MainFrame:RegisterEvent("PLAYER_REGEN_ENABLED");
        end
    else
        SwapQueue = nil; -- Clear any previously queued function
        JDI[func](self);
    end
end

function JDI:SwapToLance()
    local link, equipt = JDI:FindLance();
    if (link and equipt) then
        return; -- Nothing to do.
    end

    if (link) then
        if (JDI.Vars.Dismount and IsMounted()) then
            Dismount();
        end
        EquipItemByName(link, JDI.MAINHANDSLOT);
    else
        JDI:Warn(JDI.Locale.NoLance, "Sound\\Spells\\PVPFlagTaken.wav");
    end
end

function JDI:SwapToWeapons()
    local link = GetInventoryItemLink("player", JDI.MAINHANDSLOT);
    if ((link and JDI:IsLance(link)) or (link == nil)) then
        local wpnlink, reason = JDI:FindWeapon(JDI.Vars.MainHand, JDI.MAINHANDSLOT);
        if (wpnlink and reason) then
            -- Do nothing, already equipped.
        elseif (wpnlink) then
            EquipItemByName(JDI.Vars.MainHand, JDI.MAINHANDSLOT);
        else
            if (reason == 1) then
                JDI:Warn(JDI.Locale.WeaponNotSet, "Sound\\Spells\\PVPFlagTaken.wav");
            elseif (reason == 2) then
                JDI:Warn(format(JDI.Locale.WeaponNotFound, (GetItemInfo(JDI.Vars.MainHand))), "Sound\\Spells\\PVPFlagTaken.wav");
            end
        end

        wpnlink, reason = JDI:FindWeapon(JDI.Vars.OffHand, JDI.OFFHANDSLOT);
        if (wpnlink and reason) then
            -- Do nothing, already equipped.
        elseif (wpnlink) then
            EquipItemByName(JDI.Vars.OffHand, JDI.OFFHANDSLOT);
        else
            if (reason == 1) then
                -- Do nothing, didn't swap off hand.
            elseif (reason == 2) then
                JDI:Warn(format(JDI.Locale.WeaponNotFound, (GetItemInfo(JDI.Vars.OffHand))), "Sound\\Spells\\PVPFlagTaken.wav");
            end
        end
    end
end

function JDI:SetEnable()
    JDI.Vars.Enable = not JDI.Vars.Enable;
    if (JDI.Vars.Enable) then
        MainFrame:RegisterEvent("UI_ERROR_MESSAGE");
        MainFrame:RegisterEvent("UNIT_EXITED_VEHICLE");
        if (JDI.Vars.Combat) then
            MainFrame:RegisterEvent("PLAYER_REGEN_DISABLED");
        end
    else
        MainFrame:UnregisterEvent("UI_ERROR_MESSAGE");
        MainFrame:UnregisterEvent("UNIT_EXITED_VEHICLE");
        if (JDI.Vars.Combat) then
            MainFrame:UnregisterEvent("PLAYER_REGEN_DISABLED");
        end
    end
end

function JDI:SetOptCombat()
    JDI.Vars.Combat = not JDI.Vars.Combat;
    if (JDI.Vars.Enable and JDI.Vars.Combat) then
        MainFrame:RegisterEvent("PLAYER_REGEN_DISABLED");
    else
        MainFrame:UnregisterEvent("PLAYER_REGEN_DISABLED");
    end
end

function JDI:SetOptDismount()
    JDI.Vars.Dismount = not JDI.Vars.Dismount;
end

--[[
    JDI:FindLance
    returns (itemlink, true) if in mainhand already
    returns (itemlink, false) if in bags
    returns nil if not found
  ]]
function JDI:FindLance()
    local link = GetInventoryItemLink("player", JDI.MAINHANDSLOT);

    if (JDI:IsLance(link)) then
        return link, true;
    end
    
    for i = 0, NUM_BAG_SLOTS do
        for j = 1, GetContainerNumSlots(i) do
            link = GetContainerItemLink(i, j);
            if (JDI:IsLance(link)) then
                return link, false;
            end
        end
    end
    
    return nil;
end

--[[
    Same returns as JDI:FindLance
    except returns (false, reason) if not found
    reason: 1 MainHand not set
          : 2 Not in bags
  ]]
function JDI:FindWeapon(weapon, slot)
    local findname;
    if (weapon) then
        findname = GetItemInfo(weapon);
    end
    if (not findname) then
        return false, 1;
    end
    
    local link = GetInventoryItemLink("player", slot);
    local name;
    if (link) then
        name = GetItemInfo(link);
    end

    if (name == findname) then
        return link, true;
    end
    
    for i = 0, NUM_BAG_SLOTS do
        for j = 1, GetContainerNumSlots(i) do
            link = GetContainerItemLink(i, j);
            if (link) then
                name = GetItemInfo(link);
                if (name == findname) then
                    return link, false;
                end
            end
        end
    end
    
    return false, 2;
end

function JDI:IsLance(itemlink)
    if ((not itemlink) or (strlen(itemlink) == 0)) then
        return false;
    end

    local itemid = tonumber((select(2,strsplit(":", itemlink))));

    -- Add any new lances here. If many lances are added, consider making a table.
    if (itemid == 46069 or itemid == 46070 or itemid == 46106) then
        return true;
    end
    return false;
end

local LastWarn = "";
local LastWarnTime = -2;
function JDI:Warn(msg, sound)
    if ((msg ~= LastWarn) or (GetTime() - 2 >= LastWarnTime)) then
        LastWarn = msg;
        LastWarnTime = GetTime();
        RaidNotice_AddMessage(RaidBossEmoteFrame, msg, ChatTypeInfo["RAID_WARNING"]);
        PlaySoundFile(sound);
    end
end

SlashCmdList['JDI_GUI'] = JDI.GUI.MainFrame.Toggle;

_G['SLASH_JDI_GUI1'] = '/joustdoit';
_G['SLASH_JDI_GUI2'] = '/joust';
